@@ -36,48 +36,57 @@ public class MainWindow {
 	[Glade.Widget] Notebook view_notebook;
 	[Glade.Widget] ScrolledWindow tag_selection_scrolled;
 
-	//
-	// Menu items
-	//
+	// File
 	[Glade.Widget] MenuItem version_menu_item;
 	[Glade.Widget] MenuItem create_version_menu_item;
 	[Glade.Widget] MenuItem delete_version_menu_item;
 	[Glade.Widget] MenuItem rename_version_menu_item;
-
-	[Glade.Widget] MenuItem delete_selected_tag;
-	[Glade.Widget] MenuItem edit_selected_tag;
-
-	[Glade.Widget] MenuItem attach_tag_to_selection;
-	[Glade.Widget] MenuItem remove_tag_from_selection;
-
-	[Glade.Widget] MenuItem copy;
-	[Glade.Widget] MenuItem rotate_left;
-	[Glade.Widget] MenuItem rotate_right;
-	[Glade.Widget] MenuItem update_thumbnail;
-	[Glade.Widget] MenuItem delete_from_drive;
-
-	[Glade.Widget] MenuItem send_mail;
+	
 	[Glade.Widget] MenuItem export;
 	[Glade.Widget] MenuItem print;
-	[Glade.Widget] MenuItem select_none;
-	[Glade.Widget] MenuItem copy_location;
-	[Glade.Widget] MenuItem adjust_color;
-	[Glade.Widget] MenuItem exif_data;
-	[Glade.Widget] MenuItem sharpen;
-	[Glade.Widget] MenuItem remove_from_catalog;
+	[Glade.Widget] MenuItem send_mail;
 
+	// Edit
+	[Glade.Widget] MenuItem copy_location;
+	[Glade.Widget] MenuItem select_none;
+	[Glade.Widget] MenuItem rotate_left;
+	[Glade.Widget] MenuItem rotate_right;
+
+	[Glade.Widget] MenuItem adjust_color;
+	[Glade.Widget] MenuItem sharpen;
+
+	[Glade.Widget] MenuItem update_thumbnail;
+	[Glade.Widget] MenuItem delete_from_drive;
+	[Glade.Widget] MenuItem remove_from_catalog;
+	[Glade.Widget] MenuItem set_as_background;
+
+	[Glade.Widget] MenuItem attach_tag;
+	[Glade.Widget] MenuItem remove_tag;
+
+	// View
+	[Glade.Widget] MenuItem exif_data;
+	
 	[Glade.Widget] CheckMenuItem display_toolbar;
 	[Glade.Widget] CheckMenuItem display_sidebar;
 	[Glade.Widget] CheckMenuItem display_timeline;
 	[Glade.Widget] CheckMenuItem display_dates_menu_item;
 	[Glade.Widget] CheckMenuItem display_tags_menu_item;
 
-	[Glade.Widget] MenuItem set_as_background;
+	[Glade.Widget] MenuItem zoom_in;
+	[Glade.Widget] MenuItem zoom_out;
 
-	[Glade.Widget] MenuItem attach_tag;
-	[Glade.Widget] MenuItem remove_tag;
+	// Find
 	[Glade.Widget] MenuItem find_tag;
+	[Glade.Widget] CheckMenuItem find_untagged;
 	
+	// Tags
+	[Glade.Widget] MenuItem edit_selected_tag;
+	[Glade.Widget] MenuItem delete_selected_tag;
+
+	[Glade.Widget] MenuItem attach_tag_to_selection;
+	[Glade.Widget] MenuItem remove_tag_from_selection;
+	
+	// Other Widgets
 	[Glade.Widget] Scale zoom_scale;
 
 	[Glade.Widget] VPaned info_vpaned;
@@ -141,6 +150,8 @@ public class MainWindow {
 
 	const int PHOTO_IDX_NONE = -1;
 
+	private static Gtk.Tooltips toolTips = new Gtk.Tooltips ();
+
 	//
 	// Public Properties
 	//
@@ -219,6 +230,9 @@ public class MainWindow {
 		left_vbox.PackStart (info_box, false, true, 0);
 
 		query = new FSpot.PhotoQuery (db.Photos);
+		query.Changed += HandleQueryChanged;
+
+		db.Photos.ItemsChanged += HandleDbItemsChanged;
 #if SHOW_CALENDAR
 		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
 		cal.DaySelected += HandleCalendarDaySelected;
@@ -380,9 +394,10 @@ public class MainWindow {
 				view_notebook.CurrentPage = 1;
 			
 			JumpTo (icon_view.FocusCell);
-			//zoom_scale.Value = 0.0;
+			zoom_scale.Value = photo_view.NormalizedZoom;
 			break;
 		}
+		Selection.MarkChanged ();
 		UpdateToolbar ();
 	}
 	
@@ -403,6 +418,22 @@ public class MainWindow {
 		}
 	}
 
+	private void HandleDbItemsChanged (object sender, DbItemEventArgs args)
+	{
+		foreach (DbItem item in args.Items) {
+			Photo p = item as Photo;
+			if (p == null)
+				continue;
+			
+			if (write_metadata)
+				p.WriteMetadataToImage ();
+			
+		}
+		
+		if (args is TimeChangedEventArgs)
+			InvalidateViews ();
+	}
+
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
 	{
 		switch (view_notebook.CurrentPage) {
@@ -484,6 +515,12 @@ public class MainWindow {
 			return false;
 		}
 		
+		public void MarkChanged ()
+		{
+			if (Changed != null)
+				Changed (this);
+		}
+		
 		public IBrowsableItem this [int index] {
 			get {
 				switch (win.view_mode) {
@@ -535,7 +572,7 @@ public class MainWindow {
 				Changed (this);
 		}
 
-		private void HandleSelectionChanged (IBrowsableCollection collection)
+		public void HandleSelectionChanged (IBrowsableCollection collection)
 		{
 			if (win.view_mode == ModeType.IconView && Changed != null)
 				Changed (this);
@@ -546,7 +583,7 @@ public class MainWindow {
 			if (win.view_mode == ModeType.IconView && ItemsChanged != null)
 				ItemsChanged (this, args);
 		}
-		
+
 		public event IBrowsableCollectionChangedHandler Changed;
 		public event IBrowsableCollectionItemsChangedHandler ItemsChanged;
 	}
@@ -624,30 +661,24 @@ public class MainWindow {
 		Photo p = query.Photos [num];
 
 		p.AddTag (tags);
-
-		if (write_metadata)
-			p.WriteMetadataToImage ();
-
 		query.Commit (num);
 
 		foreach (Tag t in tags) {
-			Pixbuf icon = null;
+			if (t.Icon != null)
+				continue;
 
-			if (t.Icon == null) {
-				if (icon == null) {
-					// FIXME this needs a lot more work.
-					try {
-						Pixbuf tmp = FSpot.PhotoLoader.LoadAtMaxSize (query.Items [num], 128, 128);
-						icon = PixbufUtils.TagIconFromPixbuf (tmp);
-						tmp.Dispose ();
-					} catch {
-						icon = null;
-					}
-				}
-				
-				t.Icon = icon;
-				db.Tags.Commit (t);
+			// FIXME this needs a lot more work.
+			Pixbuf icon = null;
+			try {
+				Pixbuf tmp = FSpot.PhotoLoader.LoadAtMaxSize (query.Items [num], 128, 128);
+				icon = PixbufUtils.TagIconFromPixbuf (tmp);
+				tmp.Dispose ();
+			} catch {
+				icon = null;
 			}
+			
+			t.Icon = icon;
+			db.Tags.Commit (t);
 		}
 	}
 
@@ -735,13 +766,16 @@ public class MainWindow {
 
 		switch (args.Info) {
 		case (uint)TargetType.PhotoList:
+			db.BeginTransaction ();
 			foreach (int num in SelectedIds ()) {
 				AddTagExtended (num, tags);
 			}
+			db.CommitTransaction ();
 			break;
 		case (uint)TargetType.UriList:
 			UriList list = new UriList (args.SelectionData);
 			
+			db.BeginTransaction ();
 			foreach (string path in list.ToLocalPaths ()) {
 				Photo photo = db.Photos.GetByPath (path);
 				
@@ -752,6 +786,7 @@ public class MainWindow {
 				// FIXME this should really follow the AddTagsExtended path too
 				photo.AddTag (tags);
 			}
+			db.CommitTransaction ();
 			InvalidateViews ();
 			break;
 		case (uint)TargetType.TagList:
@@ -1103,9 +1138,11 @@ public class MainWindow {
 
 	public void HandleAttachTagMenuSelected (Tag t) 
 	{
+		db.BeginTransaction ();
 		foreach (int num in SelectedIds ()) {
 			AddTagExtended (num, new Tag [] {t});
 		}
+		db.CommitTransaction ();
 	}
 	
 	void HandleFindTagMenuSelected (Tag t)
@@ -1115,10 +1152,12 @@ public class MainWindow {
 
 	public void HandleRemoveTagMenuSelected (Tag t)
 	{
+		db.BeginTransaction ();
 		foreach (int num in SelectedIds ()) {
 			query.Photos [num].RemoveTag (t);
 			query.Commit (num);
 		}
+		db.CommitTransaction ();
 	}
 
 	//
@@ -1316,12 +1355,14 @@ public class MainWindow {
 			"Laurence Hygate",
 			"Alex Graveley",
 			"Todd Berman",
+			"Ben Monnahan",
 			"Vladimir Vukicevic",
 			"Aaron Bockover",
 			"Bengt Thuree",
 			"Jon Trowbridge",
 			"Joe Shaw",
 			"Nat Friedman",
+			"Ruben Vermeersch",
 			"Tambet Ingo",
 			"MOREAU Vincent",
 			"Alvaro del Castillo",
@@ -1329,6 +1370,7 @@ public class MainWindow {
 			"Alessandro Gervaso",
 			"Peter Johanson",
 			"Miguel de Icaza",
+			"Chad Files",
 			"Grahm Orr",
 			"Ewen Cheslack-Postava",
 			"Gabriel Burt",
@@ -1479,19 +1521,29 @@ public class MainWindow {
 
 	void AttachTags (Tag [] tags, int [] ids) 
 	{
+		db.BeginTransaction ();
 		foreach (int num in ids) {
 			AddTagExtended (num, tags);
 		}
+		db.CommitTransaction ();
 	}
 
 	public void HandleRemoveTagCommand (object obj, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 
+		db.BeginTransaction ();
 		foreach (int num in SelectedIds ()) {
-			query.Photos [num].RemoveTag (tags);
+			Photo p = query.Photos [num];
+
+			p.RemoveTag (tags);
+			/*
+			if (write_metadata)
+				p.WriteMetadataToImage ();
+			*/
 			query.Commit (num);
 		}
+		db.CommitTransaction ();
 	}
 
 	public void HandleEditSelectedTag (object sender, EventArgs ea)
@@ -1519,12 +1571,37 @@ public class MainWindow {
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 		if (tags.Length < 2)
 			return;
-
-		System.Array.Sort (tags, new TagRemoveComparer ());
-		string header = Mono.Posix.Catalog.GetString ("Merge the {0} selected tags?");
 		
+		// Translators, The singular case will never happen here.
+		string header = Mono.Posix.Catalog.GetPluralString ("Merge the selected tag",
+								    "Merge the {0} selected tags?", tags.Length);
 		header = String.Format (header, tags.Length);
-		string msg = Mono.Posix.Catalog.GetString("This operation will delete all but one of the selected tags.");
+
+		// If a tag with children tags is selected for merging, we
+		// should also merge its children..
+		ArrayList all_tags = new ArrayList (tags.Length);
+		foreach (Tag tag in tags) {
+			if (! all_tags.Contains (tag))
+				all_tags.Add (tag);
+			else
+				continue;
+
+			if (! (tag is Category))
+				continue;
+
+			(tag as Category).AddDescendentsTo (all_tags);
+		}
+
+		// debug..
+		tags = (Tag []) all_tags.ToArray (typeof (Tag));
+		System.Array.Sort (tags, new TagRemoveComparer ());
+
+		foreach (Tag tag in tags) {
+			System.Console.WriteLine ("tag: {0}", tag.Name);
+		}
+
+		string msg = Mono.Posix.Catalog.GetString("This operation will merge the selected tags and any sub-tags into a single tag.");
+
 		string ok_caption = Mono.Posix.Catalog.GetString ("_Merge tags");
 		
 		if (ResponseType.Ok != HigMessageDialog.RunHigConfirmation(main_window, 
@@ -1535,26 +1612,35 @@ public class MainWindow {
 									   ok_caption))
 			return;
 		
-		// The surviving tag is tags [0].  removetags will contain
-		// the tags to be merged.
+		// The surviving tag is the last tag, as it is definitely not a child of any other the
+		// other tags.  removetags will contain the tags to be merged.
+		Tag survivor = tags[tags.Length - 1];
+		
 		Tag [] removetags = new Tag [tags.Length - 1];
-		Array.Copy (tags, 1, removetags, 0, tags.Length - 1);
+		Array.Copy (tags, 0, removetags, 0, tags.Length - 1);
 
-
-		// Remove the defunct tags from all the photos and
-		// replace them with the new tag.
-		Photo [] photos = db.Photos.Query (tags);
+		// Add the surviving tag to all the photos with the other tags
+		Photo [] photos = db.Photos.Query (removetags);
 		foreach (Photo p in photos) {
-			p.RemoveTag (removetags);
-			p.AddTag (tags [0]);
-			db.Photos.Commit (p);
+			p.AddTag (survivor);
 		}
 
-		// Remove the defunct tags from the tag list.
+		// Remove the defunct tags, which removes them from the photos, commits
+		// the photos, and removes the tags from the TagStore
+		db.BeginTransaction ();
 		db.Photos.Remove (removetags);
+		db.CommitTransaction ();
+
 		UpdateTagEntryFromSelection ();
 
 		icon_view.QueueDraw ();
+
+		HandleEditSelectedTagWithTag (survivor);
+	}
+
+	void HandleAdjustTime (object sender, EventArgs args)
+	{
+		new TimeDialog (db, new PhotoList (Selection.Items));
 	}
 
 	void HandleAdjustColor (object sender, EventArgs args)
@@ -1611,7 +1697,7 @@ public class MainWindow {
 				} catch (System.Exception e) {
 					string msg = Mono.Posix.Catalog.GetString ("Error saving sharpened photo");
 					string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
-								     e.Message, photo.Name);
+								     e.Message, photo.Name.Replace ("_", "__"));
 					
 					HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
 										    Gtk.MessageType.Error, ButtonsType.Ok, 
@@ -1760,24 +1846,38 @@ public class MainWindow {
 			icon_view.ZoomChanged += HandleZoomChanged;
 			break;
 		}
+		
+		zoom_in.Sensitive = (zoom_scale.Value != 1.0);
+		zoom_out.Sensitive = (zoom_scale.Value != 0.0);
+	}
+	
+	void HandleQueryChanged (IBrowsableCollection sender)
+	{
+		if (find_untagged.Active != query.Untagged)
+			find_untagged.Active = query.Untagged;
 	}
 	
 	void HandleZoomChanged (object sender, System.EventArgs args)
 	{
 		zoom_scale.ValueChanged -= HandleZoomScaleValueChanged;
 
+		double zoom = .5;
 		switch (view_mode) {
 		case ModeType.PhotoView:
-			zoom_scale.Value = photo_view.NormalizedZoom;
+			zoom = photo_view.NormalizedZoom;
+			zoom_scale.Value = zoom;
 			break;
 		case ModeType.IconView:
-			double zoom = icon_view.Zoom;
+			zoom = icon_view.Zoom;
 			if (zoom == 0.0 || zoom == 100.0 || zoom != zoom_scale.Value)
 				zoom_scale.Value = zoom;
 
 			break;
 		}
 		
+		zoom_in.Sensitive = (zoom != 1.0);
+		zoom_out.Sensitive = (zoom != 0.0);
+		
 		zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
 	}
 
@@ -1830,11 +1930,11 @@ public class MainWindow {
 		if (e is UnauthorizedAccessException)
 			msg = String.Format (
 				Mono.Posix.Catalog.GetString ("No permission to delete the file:\n{0}"), 
-				fname);
+				fname).Replace ("_", "__");
 		else
 			msg = String.Format (
 				Mono.Posix.Catalog.GetString ("An error of type {0} ocurred when deleting the file:\n{1}"),
-				e.GetType (), fname);
+				e.GetType (), fname.Replace ("_", "__"));
 		
 		HigMessageDialog.RunHigConfirmation (
 			main_window, DialogFlags.DestroyWithParent, MessageType.Error,
@@ -1845,8 +1945,8 @@ public class MainWindow {
 	{
    		Photo[] photos = SelectedPhotos();
    		string header = Mono.Posix.Catalog.GetPluralString ("Delete the selected photo permanently?", 
-								 "Delete the {0} selected photos permanently?", 
-								 photos.Length);
+								    "Delete the {0} selected photos permanently?", 
+								    photos.Length);
 		header = String.Format (header, photos.Length);
 		string msg = Mono.Posix.Catalog.GetPluralString ("This deletes all versions of the selected photo from your drive.", 
 								 "This deletes all versions of the selected photos from your drive.", 
@@ -1874,8 +1974,8 @@ public class MainWindow {
 	{
    		Photo[] photos = SelectedPhotos();
    		string header = Mono.Posix.Catalog.GetPluralString ("Remove the selected photo from F-Spot?",
-								 "Remove the {0} selected photos from F-Spot?", 
-								 photos.Length);
+								    "Remove the {0} selected photos from F-Spot?", 
+								    photos.Length);
 
 		header = String.Format (header, photos.Length);
 		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the F-Spot catalog all tag information will be lost. The photos remain on your computer and can be imported into F-Spot again.");
@@ -1934,7 +2034,7 @@ public class MainWindow {
 
 		string header;
 		if (tags.Length == 1)
-			header = String.Format (Mono.Posix.Catalog.GetString ("Delete tag \"{0}\"?"), tags [0].Name);
+			header = String.Format (Mono.Posix.Catalog.GetString ("Delete tag \"{0}\"?"), tags [0].Name.Replace ("_", "__"));
 		else
 			header = String.Format (Mono.Posix.Catalog.GetString ("Delete the {0} selected tags?"), tags.Length);
 		
@@ -1957,7 +2057,7 @@ public class MainWindow {
 				string error_msg = Mono.Posix.Catalog.GetString ("Tag is not empty");
 				string error_desc = String.Format (Mono.Posix.Catalog.GetString ("Can not delete tags that have tags within them.  " + 
 												 "Please delete tags under \"{0}\" first"),
-								   e.Tag.Name);
+								   e.Tag.Name.Replace ("_", "__"));
 				
 				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
 									    Gtk.MessageType.Error, ButtonsType.Ok, 
@@ -2040,11 +2140,19 @@ public class MainWindow {
 	}
 	
 	void HandleFindUntagged (object sender, EventArgs args) {
-		tag_selection_widget.SelectionChanged -= OnTagSelectionChanged;
-		tag_selection_widget.TagSelection = new Tag [] {};
-		tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
+		if (query.Untagged == find_untagged.Active)
+			return;
 
-		query.Untagged = true;
+		if (query.Untagged)
+			query.Untagged = false;
+		else {
+			// Having tags selected wouldn't make sense
+			tag_selection_widget.SelectionChanged -= OnTagSelectionChanged;
+			tag_selection_widget.TagSelection = new Tag [] {};
+			tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
+
+			query.Untagged = true;
+		}
 	}
 	
 	void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
@@ -2339,6 +2447,7 @@ public class MainWindow {
 			       default_category = selection [0].Category;
 	       }
 
+		db.BeginTransaction ();
 	       for (int i = 0; i < tagnames.Length; i ++) {
 		       if (tagnames [i].Length == 0)
 			       continue;
@@ -2359,6 +2468,7 @@ public class MainWindow {
 		       foreach (int num in selected_photos)
 			       AddTagExtended (num, tags);
 	       }
+		db.CommitTransaction ();
 	       
 	       // Remove any removed tags from the selected photos
 	       foreach (string tagname in selected_photos_tagnames) {
@@ -2505,4 +2615,9 @@ public class MainWindow {
 				return true;
 		return false;
 	}
+
+	public static void SetTip (Widget widget, string tip)
+	{
+		toolTips.SetTip (widget, tip, tip);
+	}
 }
